<?php

/**
 * JoomlaWatch - A real-time ajax joomla monitor and live stats
 * @package JoomlaWatch
 * @version 1.2.18 BETA
 * @revision 507
 * @license http://www.gnu.org/licenses/gpl-3.0.txt 	GNU General Public License v3
 * @copyright (C) 2011 by Matej Koval - All rights reserved!
 * @website http://www.codegravity.com
 **/

/** ensure this file is being included by a parent file */
if (!defined('_JEXEC') && !defined('_VALID_MOS'))
    die('Restricted access');

class JoomlaWatchConfig {

    var $database;
    var $liveSiteCached;
    var $env;

    function JoomlaWatchConfig($database) {
        $this->env = JoomlaWatchEnvFactory::getEnvironment();
        $this->database = $database;
    }

    function getAdminitratorDirectoryName() {
        return $this->env->getAdminDir();
    }

    function getJavascriptDirectoryName() {
        return "components/com_joomlawatch/js";
    }

    /**
     * config
     */
    function checkPermissions() {
        if (!$this->isPermitted()) {
            die("Access denied. Hacking attempt has been logged and reported.");
        }
    }

    function isPermitted() {
        $rand = JoomlaWatchConfig::getRand();
        if (!$rand || $rand != addslashes(strip_tags(@ JoomlaWatchHelper::requestGet('rand')))) {
            return false;
        }
        return true;
    }


    /**
     * config
     *
     * @return unknown
     */
    function getRand() {
        $query = sprintf("select value from #__joomlawatch_config where name = 'rand' order by id desc limit 1; ");
        $rand = $this->database->resultQuery($query);
        return $rand;

    }

    /**
     * config
     */
    function isIgnored($name, $key) {
        if (!@$key) {
            return false;
        }
        $name = strtoupper($name);
        $query = sprintf("select value from #__joomlawatch_config where name='%s' limit 1", $this->database->getEscaped("JOOMLAWATCH_IGNORE_".$name));
        $rowValue = $this->database->resultQuery($query);
        $exploded = explode("\n", $rowValue);
        foreach ($exploded as $value) {
            if (JoomlaWatchHelper::wildcardSearch(trim($value), $key)) {
                return true;
            }
        }
        return false;
    }


    /**
     * config
     */
    function updateHelperCountByKey($key, $value) {
        $count = $this->getCountByKey($key);

        if (@ $count) {
            $query = sprintf("update #__joomlawatch_config set value = '%s' where (name = '%s' and date = '%d')", $this->database->getEscaped($value), $this->database->getEscaped($key), (int) $value);
            $this->database->executeQuery($query);
        } else {
            $query = sprintf("insert into #__joomlawatch_config values ('', '%s', '%s')", $this->database->getEscaped($key), $this->database->getEscaped($value));
            $this->database->executeQuery($query);
        }
    }

    /**
     * config
     */

    function getConfigValue($key) {

        $query = sprintf("select value from #__joomlawatch_config where name = '%s' limit 1", $this->database->getEscaped($key));
        $value = $this->database->resultQuery($query);
        // explicit off for checkboxes
        if ($value == "Off") {
            return false;
        }
        if ($value) {
            return addslashes($value);
        }

        $value = @ constant($key);
        return $value;
    }

    /**
     * config
     */
    function saveConfigValue($key, $value) {
        $query = sprintf("select count(name) as count from #__joomlawatch_config where name = '%s' limit 1", $this->database->getEscaped($key));
        $count = $this->database->resultQuery($query);

        if ($count) { //update
            $query = sprintf("update #__joomlawatch_config set value = '%s' where name = '%s'", $this->database->getEscaped($value), $this->database->getEscaped($key));
            $this->database->executeQuery($query);
        } else { //insert
            $query = sprintf("insert into #__joomlawatch_config values ('','%s','%s')", $this->database->getEscaped($key), $this->database->getEscaped($value));
            $this->database->executeQuery($query);
        }

    }

    /**
     * config
     */
    function removeConfigValue($key, $value) {
        $query = sprintf("delete from #__joomlawatch_config where name = '%s' limit 1", $this->database->getEscaped($key));
        $count = $this->database->resultQuery($query);
    }

    /**
     * config
     */
    function getLanguage() {
        $language = $this->getConfigValue("JOOMLAWATCH_LANGUAGE");
        return $language;
    }

    /**
     * config
     */
    function checkLicenseAccepted() {
        $accepted = $this->getConfigValue("JOOMLAWATCH_LICENSE_ACCEPTED");
        if (@ $accepted) {
            return true;
        }
        return false;
    }

    function getLicenseFilePath() {
        $config = new JConfig();
        $fileName = md5($this->getLiveSite());
        return $config->tmp_path.DS.$fileName.".tmp";
    }

    /* function createLicenseFile() {
        $result = 0;
        if (!file_exists($this->getLicenseFilePath())) {
            $ourFileHandle = fopen($this->getLicenseFilePath(), 'w') or die("can't write to temp directory: ".$config->tmp_path);
            $result = fwrite($ourFileHandle, JoomlaWatchHelper::getServerTime());
            fclose($ourFileHandle);
        } 
        return $result;
    }*/

    function useTrial() {
        if (!$this->isTrial()) {
            $this->saveConfigValue("JOOMLAWATCH_TRIAL_TIME", JoomlaWatchHelper::getServerTime());
        }
    }

    function isTrial() {
        if ($this->getConfigValue("JOOMLAWATCH_TRIAL_TIME")) {
            return true;
        }
        return false;
    }

    function isTrialTimeOver() {
        return (!($this->getTrialVersionTimeLeft() > 0));
    }

    /**
     * config
     */
    function setLicenseAccepted() {
        // $this->createLicenseFile(); not used yet
        $this->saveConfigValue("JOOMLAWATCH_LICENSE_ACCEPTED", "1");
        $this->setRand();
    }

    /**
     * config
     */
    function setLiveSite($liveSite) {
        $this->saveConfigValue("JOOMLAWATCH_LIVE_SITE", "$liveSite");
    }

    /**
     * config
     */
    function getCheckboxValue($key) {
        $setting = $this->getConfigValue($key);
        if ($setting == '1' || strtolower($setting) == 'on') {
            return true;
        }
        return false;
    }

    /**
     * If on HTTPS site, use the https links
     * @param  $originalLink
     * @return mixed
     */
    function replaceHttpByHttps($originalLink) {
        if ($this->env->isSSL()) {
            return str_ireplace("http://","https://", $originalLink);
        }
        return str_ireplace("https://","http://", $originalLink);
    }

    function getLiveSite() {
        if ($this->liveSiteCached) {
            return $this->liveSiteCached;
        }
        if (!defined('JOOMLAWATCH_LIVE_SITE')) {
            $liveSite = JoomlaWatchConfig::replaceHttpByHttps($this->getConfigValue('JOOMLAWATCH_LIVE_SITE'));
            $this->liveSiteCached = $liveSite;
            return $liveSite;
        } else {
            $liveSite = JoomlaWatchConfig::replaceHttpByHttps(rtrim(constant('JOOMLAWATCH_LIVE_SITE')));
            $this->liveSiteCached = $liveSite;
            return $liveSite;
        }
    }

    function checkIfLiveSiteMatches() {
        //TODO
    }

    //TODO
    function getAdministratorPath() {
        //TODO -> should be changed, there can be other directory
        return $this->getLiveSite().$this->getAdminitratorDirectoryName()."/";
    }

    function getAdministratorIndex() {
        //TODO -> should be changed, there can be other directory
        return $this->getLiveSite().$this->getAdminitratorDirectoryName()."/index.php";
    }


    function cleanUrl($domain) {
        $domain = str_replace("http://","",$domain);
        $domain = str_replace("https://","",$domain);
        $domain = str_replace("www.","",$domain);
        return $domain;
    }

    /**
     * Validate whether string is an IP address
     * @param  $string
     * @return bool
     */
    function isIPAddress($string) {
        $regexp = '/^((1?\d{1,2}|2[0-4]\d|25[0-5])\.){3}(1?\d{1,2}|2[0-4]\d|25[0-5])$/';
        if (preg_match($regexp, $string)) {
            return true;
        }
        return false;
    }

    function getDomainFromLiveSite() {
        $parsedUrl = @ parse_url(@$this->getLiveSite());
        $domainWithSubdomain = trim($this->cleanUrl(@$parsedUrl[host]));

        /* if it's an IP address */
        if ($this->isIPAddress($domainWithSubdomain)) {
            return $domainWithSubdomain;
        }

        /** should extract only domain, not subdomain */
        //        preg_match('/^((.+)\.)?([A-Za-z][0-9A-Za-z\-]{1,63})\.(co\.uk|me\.uk|org\.uk|com|org|net|int|eu)(\/.*)?$/', $domainWithSubdomain, $matches);
        //        return @$matches[0];

        $splittedDomain = explode(".", $domainWithSubdomain);
        $size = sizeof($splittedDomain);

        if ($size <= 1) {
            return $splittedDomain[0]; // if it's localhost or just some hostname
        }

        // by Eman Borg:
        // co.uk, com.br, com.pl fix: 
        // if middle domain name is less than 3 chars, we assume it's 1st level domain, 
        if (strlen($splittedDomain[$size-2])<=3) {
            return $splittedDomain[$size-3].".".$splittedDomain[$size-2].".".$splittedDomain[$size-1];
        }
        else {
            return $splittedDomain[$size-2].".".$splittedDomain[$size-1];
        }
    }



    /**
     * config
     */
    function isAdFree() {
        $key = md5(strrev($this->getDomainFromLiveSite()));
        if ($key == $this->getConfigValue("JOOMLAWATCH_ADFREE")) {
            return true;
        }
        return false;
    }

    /**
     * config
     */
    function isFree() {
        if ($this->getConfigValue("JOOMLAWATCH_FREE")) {
            return true;
        }
        return false;
    }

    /**
     * config
     */
    function activate($value) {
        $this->saveConfigValue('JOOMLAWATCH_ADFREE', $this->database->getEscaped($value));
        $this->saveConfigValue('JOOMLAWATCH_FRONTEND_HIDE_LOGO', "on");
        $this->saveConfigValue('JOOMLAWATCH_FRONTEND_NOFOLLOW', "on");
        $this->saveConfigValue('JOOMLAWATCH_FRONTEND_NO_BACKLINK', "on");
        if ($this->isAdFree()) {
            echo("<span style='color: green'>"._JW_CONFIG_LICENSE_ACTIVATED."</span>");
            $this->saveConfigValue('JOOMLAWATCH_FREE', 0);
        } else {
            echo("<span style='color: red'>"._JW_CONFIG_LICENCE_DONT_MATCH."</span>");
        }
    }


    /**
     * config
     */
    function useFreeVersion() {
        $this->saveConfigValue('JOOMLAWATCH_FREE', (int) 1);
    }

    /**
     * Important when doing an upgrade to data between versions, we need to know to which version these data belong
     * @return void
     */
    function saveVersionIntoDatabase() {
        $this->saveConfigValue('JOOMLAWATCH_VERSION', JOOMLAWATCH_VERSION);
    }


    function setRand() {
        $rand = md5(md5(mt_rand())+mt_rand());
        $query = sprintf("INSERT INTO #__joomlawatch_config (id, name, value) values ('', 'rand', '%s') ",$this->database->getEscaped($rand));
        $this->database->executeQuery(trim($query));
    }

    function checkLiveSite() {
        if ($this->getLiveSite() == $this->env->getRootSite()) {
            return true;
        }
        return false;
    }

    function getTrialVersionTimeLeft() {
        return (int) (16 - ((JoomlaWatchHelper::getServerTime() - $this->getConfigValue("JOOMLAWATCH_TRIAL_TIME")) / 3600 / 24) - 0.01); //because it will display 15 days as time left
    }

    function getLiveSiteWithSuffix() {
        return $this->getLiveSite().$this->env->getEnvironmentSuffix();
    }

    function renderLink($task = "", $otherParams = "") {
        return $this->env->renderLink($task, $otherParams);
    }

    function getEnvironment() {
        return get_class($this->env);
    }
}

?>