<?php

/**
 * JoomlaWatch - A real-time ajax joomla monitor and live stats
 * @package JoomlaWatch
 * @version 1.2.18 BETA
 * @revision 507
 * @license http://www.gnu.org/licenses/gpl-3.0.txt 	GNU General Public License v3
 * @copyright (C) 2011 by Matej Koval - All rights reserved!
 * @website http://www.codegravity.com
 **/

/** ensure this file is being included by a parent file */
if (!defined('_JEXEC') && !defined('_VALID_MOS'))
    die('Restricted access');

class JoomlaWatchFlowHTML {

    var $flow;
    var $visit;

    const TRUNCATE_LEN = 50;

    function JoomlaWatchFlowHTML($flow) {
        $this->flow = $flow;
        $this->visit = new JoomlaWatchVisit();
    }


    /** presentation */
    function renderEdgeJs($flowId) {
        $offset = 1/4;
        $rows = $this->flow->getFlowById($flowId);
        $sum = $this->flow->getSumOfOutgoingFlow($rows->from, $rows->to);
        $ratio = $rows->count / $sum;

        $hue = 1 - ($ratio*3/4 + $offset);
        $rgb = $this->flow->HSV_TO_RGB($hue, 1, 1);
        if ($rgb) {
            $color = sprintf("#%02x%02x%02x", $rgb['R'], $rgb['G'], $rgb['B']);
        }

        return sprintf("graph.newEdge(node_%d, node_%d, {color: '%s'}, %.2f); // lightness: %d percent: %d count; $rows->count, sum: $sum\n", $rows->from, $rows->to, $color, ($rows->count / $sum)*100, ($rows->count / $sum)*100, ($rows->count / $sum)*100, $rows->count, $sum);
    }

    /** presentation */
    function renderEdgeTableRow($flowId, $tableRowStyle) {
        $offset = 1/4;
        $rows = $this->flow->getFlowById($flowId);
        $sum = $this->flow->getSumOfOutgoingFlow($rows->from, $rows->to);
        $ratio = $rows->count / $sum;

        $output = "";

        $hue = 1 - ($ratio*3/4 + $offset);
        $rgb = $this->flow->HSV_TO_RGB($hue, 1, 1);
        if ($rgb) {
            $color = sprintf("#%02x%02x%02x", $rgb['R'], $rgb['G'], $rgb['B']);
        }
        $fromUri = $this->visit->getTitleByUriId($rows->from);
        $fromTitle = $this->visit->getUriNameByUriId($rows->from);
        $toUri = $this->visit->getTitleByUriId($rows->to);
        $toTitle = $this->visit->getUriNameByUriId($rows->to);

        $output .= sprintf("<tr>
        <td style='background-color: "."%s"."'></td>
        <th class='%s'>%.2f%%</th>
        <td class='%s' title='%s'>%s</td>
        <td class='%s' title='%s'>%s</td>
        <th class='%s'>%d</th>
        <th class='%s'>%d</th>
    </tr>", $color,
                           $tableRowStyle,
            (($rows->count / $sum)*100),
                           $tableRowStyle,
                           JoomlaWatchHelper::truncate($fromUri, self::TRUNCATE_LEN),
                           JoomlaWatchHelper::truncate($fromTitle, self::TRUNCATE_LEN),
                           $tableRowStyle,
                           JoomlaWatchHelper::truncate($toUri, self::TRUNCATE_LEN),
                           JoomlaWatchHelper::truncate($toTitle, self::TRUNCATE_LEN),
                           $tableRowStyle,
                           $rows->count,
                           $tableRowStyle, $sum);

        return $output;
    }

    /** presentation logic */
    function renderPagesCombobox($selectedUriId) {

        if (!$selectedUriId) {
            $selectedUriId = $this->visit->getMaxCountUriId();
        }

        $output = "<select name='uriId' onchange='this.form.submit();'>";

        $rows = $this->flow->getMostPopularUriIdsAndTitles();

        if ($rows)
            foreach ($rows as $row) {
                if ($row->id == $selectedUriId) {
                    $selected = "selected";
                } else {
                    $selected = "";
                }
                $uri = JoomlaWatchHelper::truncate($this->visit->getUriNameByUriId($row->id), self::TRUNCATE_LEN); //TODO get config value should be not called statically
                $title = JoomlaWatchHelper::truncate($this->visit->getTitleByUriId($row->id), self::TRUNCATE_LEN);
                if (@$uri) {
                    $output .= "<option value='".$row->id."' $selected>".$uri." (".$title.")</option>";
                }
            }

        $output .= "</select>";


        return $output;
    }


    /** presentation logic
     * UNUSED
     * @return string
     */
    function renderMostPopularUriReport() {

        $rows = $this->flow->getMostPopularUriIdsAndTitles();

        foreach ($rows as $row) {

            $output .= $this->renderFlowTable($row->id, 5, 20);
        }

        return $output;
    }


    /** presentation logic */
    function renderOutgoingLinksCountCombobox($selectedOutgoingLinksCount) {

        $output = "<select name='outgoingLinksCount' onchange='this.form.submit();'>";

        for ($i=1; $i <= 10; $i++) {
            if ($i == $selectedOutgoingLinksCount) {
                $selected = "selected";
            } else {
                $selected = "";
            }
            $output .= "<option value='".$i."' $selected>".$i." </option>";
        }

        $output .= "</select>";

        return $output;
    }

    /** presentation logic */
    function renderNestingLevelCombobox($selectedNestingLevel) {

        $output = "<select name='nestingLevel' onchange='this.form.submit();'>";

        for ($i=0; $i <= 3; $i++) {
            if ($i == (int) $selectedNestingLevel) {
                $selected = "selected";
            } else {
                $selected = "";
            }
            $output .= "<option value='".$i."' $selected>".(int) $i." </option>";
        }

        $output .= "</select>";

        return $output;
    }

    /** presentation logic */
    function renderGraph($uriId, $outgoingLinks, $nestingLevel) {

        $nodeArray = array();
        $edgeArray = array();
        $this->flow->retrieveFlow($uriId, $nodeArray, $edgeArray, $outgoingLinks, $nestingLevel);

        $edgesOutput = "";
        $nodesOutput = "";

        // print_r($nodeArray);
        foreach ($nodeArray as $key => $value) {
            $nodesOutput .=sprintf("var node_%d = graph.newNode({label: '%s', sublabel: '%s'});\n", $value,
                                   addslashes(htmlspecialchars_decode(JoomlaWatchHelper::truncate($this->visit->getTitleByUriId($value), self::TRUNCATE_LEN))),
                                   addslashes(htmlspecialchars_decode(JoomlaWatchHelper::truncate($this->visit->getUriNameByUriId($value), self::TRUNCATE_LEN))));
        }
        foreach ($edgeArray as $key => $value) {
            $edgesOutput .= $this->renderEdgeJs($value);
        }

        return $nodesOutput.$edgesOutput;
    }

    function renderFlowTable($selectedUriId, $outgoingLinks, $nestingLevel) {

        $nodeArray = array();
        $edgeArray = array();
        $this->flow->retrieveFlow($selectedUriId, $nodeArray, $edgeArray,  $outgoingLinks, $nestingLevel);

        $output = "<table border='0' width='100%'><tr>
        <th class='tableRow0'>Color</th>
        <th class='tableRow0'>Percentage</th>
        <th class='tableRow0'>From</th>
        <th class='tableRow0'>To</th>
        <th class='tableRow0'>Count</th>
        <th class='tableRow0'>Sum</th>
        ";

        $i = 1;
        foreach ($edgeArray as $key => $value) {
            $tableRowStyle = "tableRow".(int) ($i%2);
            $output .= $this->renderEdgeTableRow($value, $tableRowStyle);
            $i++;
        }

        $output .= "</table>";


        return $output;
    }

}
?>